﻿local IM = InterruptManager
local IMversion = IM.GetVersion()

function IM.OpenConfig()
    if not InterruptManagerConfig then
        IM.CreateConfig()
        InterruptManagerConfig:Show()
        else
        InterruptManagerConfig:Show()
    end
    InterruptManagerAnchor:Show()
    InterruptManagerAnchor:SetHeight(IMDB.anchorSize[2])
    for i=1,5 do
        getglobal("InterruptManagerStatusBar"..i):Show()
        getglobal("InterruptManagerIcon"..i):Show()
        if (IMDB.interrupters[i].name ~= "") then
            getglobal("InterruptManagerConfigEditbox"..i):SetText(IMDB.interrupters[i].name)
        end
    end
    SendAddonMessage("InterruptManager","Version"..IMversion,"RAID")
end

function IM.CreateConfig()
    -- Configuration box
    f = CreateFrame("Frame","InterruptManagerConfig")
    f:SetHeight(460)
    f:SetWidth(300)
    f:SetPoint("CENTER")
    f:SetParent(UIParent)
    f:SetFrameLevel(1)
    f:SetMovable(1)
    f:SetBackdrop({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\AddOns\\InterruptManager\\Textures\\ConfigBorder",
        tile = true,
        tileSize = 16,
        edgeSize = 16,
        insets = {left = 5,right = 5,top = 5,bottom = 5}
    })
    
    -- Configuration box title
    f = CreateFrame("Frame")
    f:SetHeight(30)
    f:SetWidth(200)
    f:SetPoint("CENTER","InterruptManagerConfig","TOP",0,-15)
    f:SetParent(InterruptManagerConfig)
    f:SetScript("OnMouseDown",function()InterruptManagerConfig:StartMoving()end)
    f:SetScript("OnMouseUp",function()InterruptManagerConfig:StopMovingOrSizing()end)
    f:SetBackdrop({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\AddOns\\InterruptManager\\Textures\\ConfigBorder",
        tile = true,
        tileSize = 16,
        edgeSize = 16,
        insets = {left = 5,right = 5,top = 5,bottom = 5}
    })
    t = f:CreateFontString()
    t:SetPoint("CENTER",f,"CENTER",0,0)
    t:SetFont("Fonts\\FRIZQT__.TTF",12)
    t:SetText("Interrupt Manager")
    
    local moveAll = 30
    -- Close button
    IM.CreateConfigButton("Close",80,0,-230+moveAll,IM.OnCloseClick)
    -- Reset button
    IM.CreateConfigButton("Reset",80,0,0+moveAll,IM.Reset)
    -- Editboxes
    IM.CreateConfigEditbox("First Interrupter",0,150+moveAll)
    IM.CreateConfigEditbox("Second Interrupter",0,120+moveAll)
    IM.CreateConfigEditbox("Third Interrupter",0,90+moveAll)
    IM.CreateConfigEditbox("Fourth Interrupter",0,60+moveAll)
    IM.CreateConfigEditbox("Fifth Interrupter",0,30+moveAll)
    -- Fill in name buttons
    IM.CreateConfigButton("1",20,-115,150+moveAll,function() IM.FillInName(1) end)
    IM.CreateConfigButton("2",20,-115,120+moveAll,function() IM.FillInName(2) end)
    IM.CreateConfigButton("3",20,-115,90+moveAll,function() IM.FillInName(3) end)
    IM.CreateConfigButton("4",20,-115,60+moveAll,function() IM.FillInName(4) end)
    IM.CreateConfigButton("5",20,-115,30+moveAll,function() IM.FillInName(5) end)
    -- Checkboxes
    IM.CreateConfigCheckbutton("Lock bars",-80,-30+moveAll,"Turn on to lock bars",1-IMDB.anchorMovable,IM.OnFrameLock)
    IM.CreateConfigCheckbutton("Solo mode",-80,-60+moveAll,"Turn on to be warned when your target/focus is casting, regardless of your position in the queue",IMDB.soloMode,IM.OnSoloModeToggle)
    IM.CreateConfigCheckbutton("Announce",-80,-90+moveAll,"Turn on to announce your interrupts",IMDB.announce,IM.OnAnnounceToggle)
    IM.CreateConfigCheckbutton("PUG mode",-80,-120+moveAll,"Turn on if someone is missing the addon",IMDB.pugMode,IM.OnPUGModeToggle)
    IM.CreateConfigCheckbutton("Watch target",-80,-150+moveAll,"Turn on to be warned when your target casts a spell",IMDB.targetWarn,IM.OnTargetWatchToggle)
    IM.CreateConfigCheckbutton("Watch focus ",-80,-180+moveAll,"Turn on to be warned when your focus casts a spell",IMDB.focusWarn,IM.OnFocusWatchToggle)
    InterruptManagerConfig:Hide()
end

function IM.CreateConfigCheckbutton(text,x,y,tooltip,checked,script)
    for i=1,100 do
        if not getglobal("InterruptManagerConfigCheckbutton"..i) then
            f = CreateFrame("CheckButton","InterruptManagerConfigCheckbutton"..i,InterruptManagerConfig,"ChatConfigCheckButtonTemplate")
            f:SetSize(20,20)
            f:SetPoint("CENTER","InterruptManagerConfig","CENTER",x,y)
            f:SetFrameLevel(2)
            f:SetScript("OnClick",script)
            f:SetChecked(checked)
            f:SetNormalTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigCheckboxUp")
            f:SetPushedTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigCheckboxDown")
            f:SetHighlightTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigButtonHighlight",0)
            f:SetCheckedTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigCheckboxChecked")
            f:SetDisabledTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigButtonDisabled")
            f.tooltip = tooltip
            t = f:CreateFontString("InterruptManagerConfigCheckButton"..i.."Text")
            t:SetPoint("LEFT","InterruptManagerConfigCheckbutton"..i,"LEFT",25,0)
            t:SetFont("Fonts\\FRIZQT__.TTF",12)
            t:SetText(text)
            break
        end
    end
end

function IM.CreateConfigEditbox(text,x,y)
    for i=1,100 do
        if not getglobal("InterruptManagerConfigEditbox"..i) then
            f = CreateFrame("Editbox","InterruptManagerConfigEditbox"..i)
            f:SetSize(200,30)
            f:SetParent(InterruptManagerConfig)
            f:SetPoint("CENTER","InterruptManagerConfig","CENTER",x,y)
            f:SetFrameLevel(2)
            f:SetBackdrop({
                bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
                edgeFile = "Interface\\AddOns\\InterruptManager\\Textures\\ConfigBorder",
                tile = true,
                tileSize = 16,
                edgeSize = 16,
                insets = {left = 5,right = 5,top = 5,bottom = 5}
            })
            f:SetAutoFocus(false)
            f:ClearFocus()
            f:SetFont("Fonts\\FRIZQT__.TTF",12)
            f:Insert(text)
            f:SetMaxLetters(20)
            f:SetTextInsets(9,0,0,0)
            f:SetScript("OnEscapePressed",function()getglobal("InterruptManagerConfigEditbox"..i):ClearFocus()end)
            f:SetScript("OnEnterPressed",function()getglobal("InterruptManagerConfigEditbox"..i):ClearFocus()end)
            f:SetScript("OnTabPressed",IM.EditboxOnTabPressed)
            f:SetScript("OnEditFocusGained",function()getglobal("InterruptManagerConfigEditbox"..i):HighlightText()end)
            f:SetScript("OnEditFocusLost",function()getglobal("InterruptManagerConfigEditbox"..i):HighlightText(0,0)end)
            break
        end
    end
end

function IM.EditboxOnTabPressed()
    for i=1,5 do
        f = getglobal("InterruptManagerConfigEditbox"..i)
        if f then
            if f:HasFocus() then
                if i < 5 then
                    getglobal("InterruptManagerConfigEditbox"..i+1):SetFocus()
                    break
                    else
                    InterruptManagerConfigEditbox1:SetFocus()
                    break
                end
            end
        end
    end
end

function IM.CreateConfigButton(text,width,x,y,script,tooltip)
    for i=1,100 do
        if not getglobal("InterruptManagerConfigButton"..i) then
            f = CreateFrame("Button","InterruptManagerConfigButton"..i)
            f:SetScript("OnClick",script)
            f:SetHeight(20)
            f:SetWidth(width)
            f:SetParent(InterruptManagerConfig)
            f:SetPoint("CENTER","InterruptManagerConfig","CENTER",x,y)
            f:SetFrameLevel(2)
            f:SetNormalTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigButtonUp")
            f:SetPushedTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigButtonDown")
            f:SetHighlightTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigButtonHighlight",0)
            f:SetDisabledTexture("Interface\\AddOns\\InterruptManager\\Textures\\ConfigButtonDisabled")
            f.tooltip = tooltip
            
            local font = f:CreateFontString()
            f:SetFontString(font)
            f:SetPushedTextOffset(-1,-1)
            font:SetFont("Fonts\\FRIZQT__.TTF",12)
            font:SetText(text)
            font:SetTextColor(0,0,0)
            break
        end
    end
end

function IM.FillInName(i)
    if UnitExists("target") then
        local name, realm = UnitName("target")
        if realm then
            getglobal("InterruptManagerConfigEditbox"..i):SetText(name.."-"..realm)
        else
            getglobal("InterruptManagerConfigEditbox"..i):SetText(name)
        end
    end
end

function IM.OnLogin()
    local db = IMDB.interrupters
    
    InterruptManagerAnchor:Hide()
    for i=1,5 do
        if db and db[i].active then
            getglobal("InterruptManagerStatusBar"..i):Show()
            getglobal("InterruptManagerIcon"..i):Show()
            InterruptManagerAnchor:Show()
            InterruptManagerAnchor:SetHeight(IMDB.anchorSize[2]*i*0.2)
            else
            getglobal("InterruptManagerStatusBar"..i):Hide()
            getglobal("InterruptManagerIcon"..i):Hide()
        end
    end
end

function IM.Reset()
    InterruptManagerConfigEditbox1:SetText("First Interrupter")
    InterruptManagerConfigEditbox2:SetText("Second Interrupter")
    InterruptManagerConfigEditbox3:SetText("Third Interrupter")
    InterruptManagerConfigEditbox4:SetText("Fourth Interrupter")
    InterruptManagerConfigEditbox5:SetText("Fifth Interrupter")
    
    for i=1,5 do
        getglobal("InterruptManagerStatusBarText"..i):SetText("")
        getglobal("InterruptManagerStatusBar"..i):SetPoint("LEFT",getglobal("InterruptManagerIcon"..i),"RIGHT")
        getglobal("InterruptManagerStatusBar"..i):SetValue(0)
        
        IMDB.interrupters[i] = {}
        IMDB.interrupters[i].active = false
        IMDB.interrupters[i].spells = {["active"] = false}
        IMDB.interrupters[i].name = ""
        IMDB.interrupters[i].ready = false
        IMDB.interrupters[i].next = false
    end
end

function IM.OnFrameLock()
    if IMDB.anchorMovable == 1 then
        IMDB.anchorMovable = 0
        ChatFrame1:AddMessage("InterruptManager: Frame locked.",1,0.5,0)
        InterruptManagerConfigCheckbutton1:SetChecked(1)
        InterruptManagerAnchor:SetMovable(false)
        InterruptManagerAnchor:EnableMouse(false)
        InterruptManagerAnchor:SetScript("OnMouseDown",nil)
        InterruptManagerAnchor:SetScript("OnMouseUp",nil)
        else
        IMDB.anchorMovable = 1
        InterruptManagerConfigCheckbutton1:SetChecked(0)
        ChatFrame1:AddMessage("InterruptManager: Frame unlocked.",1,0.5,0)
        InterruptManagerAnchor:SetMovable(true)
        InterruptManagerAnchor:EnableMouse(true)
        InterruptManagerAnchor:SetScript("OnMouseDown",function()InterruptManagerAnchor:StartMoving()end)
        InterruptManagerAnchor:SetScript("OnMouseUp",IM.SavePosition)
    end
end

function IM.OnSoloModeToggle()
    if IMDB.soloMode == 1 then
        IMDB.soloMode = 0
        ChatFrame1:AddMessage("InterruptManager: Solo mode disabled.",1,0.5,0)
        else
        IMDB.soloMode = 1
        ChatFrame1:AddMessage("InterruptManager: Solo mode enabled. You will now be warned when your target/focus starts casting a spell, regardless of your position in the queue.",1,0.5,0)
    end
end

function IM.OnAnnounceToggle()
    if IMDB.announce == 0 then
        IMDB.announce = 1
        IMDB.announceAfterPugModeOff = 1
        IMDB.pugMode = 0
        InterruptManagerConfigCheckbutton4:SetChecked(0)
        ChatFrame1:AddMessage("InterruptManager: Announce enabled.",1,0.5,0)
        else
        IMDB.announce = 0
        IMDB.announceAfterPugModeOff = 0
        ChatFrame1:AddMessage("InterruptManager: Announce disabled.",1,0.5,0)
    end
end

function IM.OnPUGModeToggle()
    if IMDB.pugMode == 0 then
        IMDB.pugMode = 1
        IMDB.announce = 0
        InterruptManagerConfigCheckbutton3:SetChecked(0)
        ChatFrame1:AddMessage("InterruptManager: PUG mode enabled. You will now announce when someone uses their interrupt ability, and whose turn it is next. This option is disabled when logging in.",1,0.5,0)
        SendAddonMessage("InterruptManager","PUG1","RAID")
        else
        IMDB.pugMode = 0
        ChatFrame1:AddMessage("InterruptManager: PUG mode disabled.",1,0.5,0)
        SendAddonMessage("InterruptManager","PUG0","RAID")
    end
end

function IM.OnTargetWatchToggle()
    if IMDB.targetWarn == 0 then
        IMDB.targetWarn = 1
        ChatFrame1:AddMessage("InterruptManager: You will now be warned when your target starts casting a spell.",1,0.5,0)
        else
        IMDB.targetWarn = 0
        ChatFrame1:AddMessage("InterruptManager: You will no longer be warned when your target starts casting a spell.",1,0.5,0)
    end
end

function IM.OnFocusWatchToggle()
    if IMDB.focusWarn == 0 then
        IMDB.focusWarn = 1
        ChatFrame1:AddMessage("InterruptManager: You will now be warned when your focus starts casting a spell.",1,0.5,0)
        else
        IMDB.focusWarn = 0
        ChatFrame1:AddMessage("InterruptManager: You will no longer be warned when your focus starts casting a spell.",1,0.5,0)
    end
end

function IM.BroadcastInterrupters()
    local msg = "rotationinfo:"
    
    for i = 1,5 do
        local text = _G["InterruptManagerConfigEditbox"..i]:GetText()
        local missing = true
        
        if (text ~= "" and not strfind(text, "Interrupter")) then
            msg = msg.._G["InterruptManagerConfigEditbox"..i]:GetText()
        end
        
        if (IsInGroup(LE_PARTY_CATEGORY_HOME)) then
            for i = 1,4 do
                if (UnitName("party"..i) == text) then
                    missing = false
                end
            end
        elseif (IsInRaid(LE_PARTY_CATEGORY_HOME)) then
            for i = 1,40 do
                if (UnitName("raid"..i) == text) then
                    missing = false
                end
            end
        end
        
        if (missing) then
            ChatFrame1:AddMessage("Warning: " .. text .. " is missing from the raid group. InterruptManager will not work as intended unless you click close while all rotation members are in your group.",1,0.5,0)
        end
    end
    
    SendAddonMessage("InterruptManager", msg, "RAID")
    SendAddonMessage("InterruptManager", "versioninfo:" .. IMversion, "RAID")
end

function IM.AddInterrupter(text, i)
    local db = IMDB.interrupters
    
    db[i].active = true
    db[i].name = text
    db[i].ready = true
    db[i].pos = i
    db[i].readyTime = i
    if (i == 1) then db[i].next = true else db[i].next = false end
end

function IM.SetNameToPosition(name, pos)
    local db = IMDB.interrupters
    
    for k,v in pairs(db) do
        if (v.name == name) then
            _G["InterruptManagerStatusBar"..k]:SetPoint("LEFT", _G["InterruptManagerIcon"..pos], "RIGHT")
        end
    end
end

function IM.UpdateBars()
    -- Reset stuff
    for i=1,5 do
        IMDB.interrupters[i] = {}
        IMDB.interrupters[i].active = false
        IMDB.interrupters[i].spells = {}
        IMDB.interrupters[i].name = ""
        IMDB.interrupters[i].ready = false
        IMDB.interrupters[i].next = false
    end
    
    local db = IMDB.interrupters
    
    for i = 1,5 do
        local text = _G["InterruptManagerConfigEditbox"..i]:GetText()
        
        if (text ~= "" and not strfind(text, " Interrupter")) then
            IM.AddInterrupter(text, i)
        end
    end
    
    InterruptManagerAnchor:SetHeight(0)
    
    for i=1,5 do
        _G["InterruptManagerStatusBar"..i]:SetPoint("LEFT", _G["InterruptManagerIcon"..i], "RIGHT")
        InterruptManagerAnchor:Hide()
        _G["InterruptManagerStatusBar"..i]:Hide()
        _G["InterruptManagerIcon"..i]:Hide()
    end
    
    for i=1,5 do
        local text = _G["InterruptManagerConfigEditbox"..i]:GetText()
        if text ~= "" and not strfind(text, " Interrupter") then
            InterruptManagerAnchor:SetHeight(0.2*i*IMDB.anchorSize[2])
            InterruptManagerAnchor:Show()
            _G["InterruptManagerStatusBar"..i]:Show()
            _G["InterruptManagerIcon"..i]:Show()
        end
    end
end

function IM.OnCloseClick()
    InterruptManagerConfig:Hide()
    IM.UpdateBars()
    
    if (IsInGroup(LE_PARTY_CATEGORY_HOME) or IsInRaid(LE_PARTY_CATEGORY_HOME)) then
        IM.BroadcastInterrupters()
    end
end

function IM.AddonMessageReceived(...)
    local msg, _, sender = select(2, ...)
    if (strfind(sender, UnitName("player"))) then return end
    
    for k,v in pairs(IM.addonMessages) do
        local key = v[1]
        local func = v[2]
        
        if (strfind(msg, key)) then
            msg = gsub(msg, key, "")
            func(msg, sender)
        end
    end
end

function IM.ReceiveRotationInfo(msg)
    -- Message example
    -- playername[,playername][,playername][,playername][,playername]
    local db = IMDB.interrupters
    
    -- Don't procceed unless player's name is in the message
    if not strfind(msg, UnitName("player")) then return end
    
    for i = 1,5 do
        if (msg ~= "") then
            local temp = strfind(",", msg)
            local name = strsub(1, temp)
            msg = gsub(msg, name..",", "")
            
            _G["InterruptManagerConfigEditbox"..i]:SetText(name)
        end
    end
    
    SendAddonMessage("InterruptManager", "versioninfo:" .. IMversion, "RAID")
    IM.SendSpellInfo()
    IM.UpdateBars()
end

function IM.ReceiveSpellInfo(spell, sender)
    -- Message example
    -- spellid
    local db = IMDB.interrupters
    
    for i = 1,5 do
        if (sender == db.name) then
            db.spells[tonumber(spell)] = GetTime()
        end
    end
end

function IM.ReceiveVersionInfo(msg)
    local otherVersion = msg
    local myVersion = IMversion
    
    if (otherVersion > myVersion) then
        ChatFrame1:AddMessage("InterruptManager: An update is available for InterruptManager.",1,0.5,0)
    end
end

function IM.SendSpellInfo()
    local class = select(2, UnitClass("player"))
    
    for k,v in pairs(IM.classSpells) do
        local key = v[1]
        local func = v[2]
        
        if (key == class) then
            func()
        end
    end
end

IM.addonMessages =
{
{"addspell:", IM.ReceiveSpellInfo},
{"rotationinfo:", IM.ReceiveRotationInfo},
{"versioninfo:", IM.ReceiveVersionInfo}
}

IM.classSpells =
{
{"WARRIOR", IM.WarriorSpells}
}

function round(num,dec)
    return string.format("%." .. dec .. "f", tostring(num))
end